header {
package antlr.util;

import antlr.*;
import antlr.collections.*;
import java.io.*;
}

/** Parse simple grammars */
class GrammarParser extends Parser;

options {
	buildAST=true;
    exportVocab=Grammar;
}

tokens {
	RULE_DEF;
	BLOCK;
	ALT;
}

{
public static void main(String[] args) {
	System.out.println(GrammarParser.toAST(args[0]).toStringList());
}

public static AST toAST(String t) {
	GrammarLexer lex = new GrammarLexer(new StringReader(t));
	GrammarParser p = new GrammarParser(lex);
	try {
		p.rules();
		return p.getAST();
	}
	catch (Exception e) {
		System.err.println("Problem with grammar: "+e);
		return null;
	}
}
}

rules	:	(rule)+
		;

rule	:	r:RULE_REF^ {#r.setType(RULE_DEF);} COLON! block SEMI!
		;

block!
		:	a1:alt {#block=#[BLOCK,"BLOCK"]; #block.addChild(#a1);}
			( OR a2:alt { #block.addChild(#a2);} )*
		;

alt!	:	{#alt=#[ALT,"ALT"];} (e:element {#alt.addChild(#e);})*
		;

element :   TOKEN_REF
        |   RULE_REF
		|	STRING
		|	ebnf
        ;

ebnf	:	LPAREN! block RPAREN!
			(	QUESTION^
			|	STAR^
			|	PLUS^
			)?
		;


class GrammarLexer extends Lexer;

options {
	charVocabulary='\03'..'\377';
	k=2;
}

SEMI	:	';'
		;

OR		:	'|';

COLON	:	':' ;

LPAREN	:	'(' ;

RPAREN	:	')' ;

QUESTION:	'?' ;

STAR	:	'*' ;

PLUS	:	'+' ;

TOKEN_REF:	'A'..'Z' ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*
		;

RULE_REF:	'a'..'z' ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*
		;

STRING	:	'"' (options {greedy=false;}:.)+ '"'
		;

NEWLINE	:	( '\r' | "\r\n" | '\n' ) {newline(); $setType(Token.SKIP);}
		;

WS		:	(' '|'\t')+ {$setType(Token.SKIP);}
		;
