header {
package antlr.util.lout;

import antlr.*;
import antlr.collections.*;
import java.io.*;
import org.antlr.stringtemplate.*;
}

/** Parse ASTs generated from simple grammars by grammar.g
 *  and generate Lout.   Use the following templates:
 *
 *  char.st             optional.st         sequence.st
 *  closure.st          posclosure.st       subrule.st
 *  rootNode.st         syndiag.st          rule.st
 *  token.st
 */
class GrammarTreeToLout extends GrammarTreeParser;

{
protected StringTemplateGroup stgroup = null;

public GrammarTreeToLout(StringTemplateGroup stgroup) {
	this();
	this.stgroup = stgroup;
}
}

rules returns [String r=null]
{
	StringBuffer buf = new StringBuffer(4000);
	String a=null;
}
		:	(a=rule {buf.append(a);})+
			{r = buf.toString();}
		;

rule returns [String r=null]
{
	String b=null;
	StringTemplate syndiag = stgroup.getInstanceOf("antlr/util/lout/syndiag");
}
		:	#(rd:RULE_DEF b=block[stgroup.getInstanceOf("antlr/util/lout/subrule")])
			{syndiag.setAttribute("title",#rd.getText());
			 syndiag.setAttribute("diag", b);
			 r=syndiag.toString();
			}
		;

block[StringTemplate blockST] returns [String r=null]
{
	StringBuffer buf = new StringBuffer(4000);
	String a=null;
	int i = 0;
	char start = 'A';
}
		:	#(  root:BLOCK
				(	{root.getNumberOfChildren()>1||blockST.getName().equals("antlr/util/lout/closure")}?
					(	a=alt
						{
						if ( blockST.getName().equals("antlr/util/lout/closure") ) {
							start='B';
						}
						blockST.setAttribute("moreThanOneAlt","true");
						blockST.setAttribute("elements",(char)(i+start)+" { "+a+" }");
						i++;
						}
					)+
					{r = blockST.toString();}
				|	{root.getNumberOfChildren()==1}?
					a=alt
					{
					//System.out.println("block name is "+blockST.getName());
					if ( blockST.getName().equals("antlr/util/lout/subrule") ) {
						r = a;
					}
					else {
						blockST.setAttribute("elements",a);
						r = blockST.toString();
					}
					}
				)
			 )
		;

alt returns [String r=null] // could return stringtemplate here
{
	StringBuffer buf = new StringBuffer(4000);
	String a=null,a2=null;
	StringTemplate sequence = stgroup.getInstanceOf("antlr/util/lout/sequence");
	int i = 0;
}
		:	#(	root:ALT {/*System.out.println("Num children=="+root.getNumberOfChildren()+
					": "+root.toStringList());*/}
				(	{root.getNumberOfChildren()>1}?
					(	a=element
						{sequence.setAttribute("elements",(char)(i+'A')+
											   " { "+a+" }\n"); i++;
						}
					)*
					{r = sequence.toString();}
				|	{root.getNumberOfChildren()==1}?
					r=element
				)
			 )
		;

element returns [String r=null]
{
	StringTemplate e = null;
}
		:   t:TOKEN_REF
			{
			e=stgroup.getInstanceOf("antlr/util/lout/token");
			e.setAttribute("label", #t.getText());
			r=e.toString();
			}
        |   rr:RULE_REF
			{
			e=stgroup.getInstanceOf("antlr/util/lout/rule");
			e.setAttribute("label", #rr.getText());
			r=e.toString();
			}
		|	s:STRING
			{
			e=stgroup.getInstanceOf("antlr/util/lout/token");
			e.setAttribute("label", #s.getText());
			r=e.toString();
			}
		|	r=ebnf
        ;

ebnf returns [String r=null]
		:	#( QUESTION r=block[stgroup.getInstanceOf("antlr/util/lout/optional")] )
		|	#( STAR r=block[stgroup.getInstanceOf("antlr/util/lout/closure")] )
		|	#( PLUS r=block[stgroup.getInstanceOf("antlr/util/lout/posclosure")] )
		|	r=block[stgroup.getInstanceOf("antlr/util/lout/subrule")]
		;
