header {
package antlr.util;

import antlr.*;
import antlr.collections.*;
import java.io.*;
}

/** Parse lisp tree strings, convert to AST. */
class LispTreeParser extends Parser;

options {
	buildAST=true;
}

{
public static AST toAST(String t) {
	LispTreeLexer lex = new LispTreeLexer(new StringReader(t));
	LispTreeParser p = new LispTreeParser(lex);
	try {
		p.tree();
		return p.getAST();
	}
	catch (Exception e) {
		return null;
	}
}
}

tree    :   LPAREN!
                (   WORD^
                |   STRING^
                )
                ( element )*
            RPAREN!
        ;

element :   atom
        |   tree
        ;

atom    :   WORD
        |   STRING
        ;

class LispTreeLexer extends Lexer;

options {
	charVocabulary='\03'..'\377';
	k=2;
}

LPAREN	:	'(' ;

RPAREN	:	')' ;

WORD  	:	(~('('|')'|'"'|' '|'\t'|'\n'|'\r'))+
		;

STRING	:	'"' (options {greedy=false;}:.)+ '"'
		;

NEWLINE	:	( '\r' | "\r\n" | '\n' ) {newline();}
		;

WS		:	(' '|'\t')+ {$setType(Token.SKIP);}
		;